﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using DG.Tweening;

public class BattleManager : MonoBehaviour
{

    public GameObject enemy;
    public GameObject ally;
    public GameObject parent;
    public GameObject platform;

    public GameObject player;

    public CameraFollow camera;
    public BattleCanvas canvas;

    List<Transform> playerList;
    List<Transform> enemyList;

    public BattleData config;
    public BattleData defaultConfig;

    public float maxRange = 20;

    public CharacterData playerData;
    public CharacterData alliesData;
    public CharacterData enemiesData;

    bool startedBattle = false;

    public Text message;
    public Image backPanel;

    public string WinMessage = "VICTORY!";
    public string LoseMessage = "DEFEAT!";

    GameObject newPlayer = null;

    void Awake()
    {
        playerList = new List<Transform>();
        enemyList = new List<Transform>();
        Reset();
    }

    public void StartBattle(BattleData cfg = null)
    {
        Reset();
        platform.SetActive(false);
        parent.SetActive(true);
        gameObject.SetActive(true);
        canvas.StartNoise();
        if (cfg == null)
        {
            config = defaultConfig;
        }
        else
        {
            config = cfg;
        }
        backPanel.color = new Color(backPanel.color.r, backPanel.color.g, backPanel.color.b,
            1);
        string nextText = cfg.change < 0 ? -1 * cfg.change + "  people joined your army after the sacrifice." 
            : cfg.change + " left your army for displeasing the gods.";
        nextText += "\n\n WASD to move | SPACE/Click to Attack";
        message.text = nextText;
        var seq = DOTween.Sequence();
        seq.Append(backPanel.DOFade(1, 0.5f));
        seq.Append(message.DOFade(1, 0.5f));
        seq.AppendInterval(2);
        seq.Append(message.DOFade(0, 0.25f));
        seq.AppendCallback(() =>
            {
                platform.SetActive(true);
                StartRealBattle();
            });
        seq.Append(backPanel.DOFade(0, 0.5f));
    }

    void StartRealBattle()
    {
        for (int i = 0; i < config.totalAllies; i++)
        {
            var ally1 = (Instantiate(ally, new Vector3(Random.Range(-maxRange, maxRange), 0, Random.Range(-maxRange, maxRange)), Quaternion.Euler(0, 90, 0)) as GameObject);
            ally1.transform.parent = transform;
            playerList.Add(ally1.transform);
        }

        newPlayer = (Instantiate(player, new Vector3(0, 0, 0), Quaternion.Euler(0, 90, 0))) as GameObject;
        newPlayer.GetComponent<PlayerController>().Init(playerData.maxSpeed,
            playerData.maxHealth, playerData.maxDamage, playerData.fireRate);
        playerList.Add(newPlayer.transform);
        camera.Init(newPlayer.transform);


        for (int j = 0; j < config.totalEnemies; j++)
        {
            var enemy1 = (Instantiate(enemy, new Vector3(Random.Range(-maxRange, maxRange), 0, Random.Range(-maxRange, maxRange)), Quaternion.Euler(0, 90, 0)) as GameObject);
            enemy1.transform.parent = transform;
            enemyList.Add(enemy1.transform);
        }

        for (int k = 0; k < playerList.Count; k++)
        {
            var u = playerList[k].GetComponent<EnemyController>();
            if (u != null)
            {
                u.Init(alliesData.maxHealth, Random.Range(alliesData.minSpeed, alliesData.maxSpeed),
                    Random.Range(alliesData.minDamage, alliesData.maxDamage), 
                    alliesData.fireRate, enemyList);
            }
        }

        for (int k = 0; k < enemyList.Count; k++)
        {
            enemyList[k].GetComponent<EnemyController>().Init(
                enemiesData.maxHealth, 
                Random.Range(enemiesData.minSpeed, enemiesData.maxSpeed),
                Random.Range(enemiesData.minDamage, enemiesData.maxDamage),
                enemiesData.fireRate, playerList);
        }

        startedBattle = true;
    }

    void Update()
    {
        if (startedBattle)
        {
           
            if (newPlayer == null)
            {
                EndBattle(false);
            }
            else
            {
                int count = enemyList.Count;
                int otherCount = 0;
                for (int i = 0; i < count; i++)
                {
                    if (enemyList[i] == null)
                    {
                        otherCount += 1;
                    }
                }
                if (otherCount == count)
                {
                    EndBattle(true);
                }
            }
        }
    }

    void EndBattle(bool won)
    {
        var seq = DOTween.Sequence();
        seq.Append(backPanel.DOFade(1, 1));
        seq.AppendInterval(1);
        if (won)
        {
            message.text = WinMessage;
        }
        else
        {
            message.text = LoseMessage;
        }
        seq.Append(message.DOFade(1, 0.5f));
        seq.AppendInterval(2);
        seq.AppendCallback(() =>
            {
                parent.SetActive(false);
                StateManager.instance.EndBattle(won);
            });
        startedBattle = false;
        gameObject.SetActive(false);
    }

    void Reset()
    {
        startedBattle = false;
        message.DOFade(0, 0.1f);
        gameObject.SetActive(false);
        for (int i = 0; i < playerList.Count; i++)
        {
            if (playerList[i] != null)
                Destroy(playerList[i].gameObject);
        }
        playerList.Clear();

        for (int i = 0; i < enemyList.Count; i++)
        {
            if (enemyList[i] != null)
                Destroy(enemyList[i].gameObject);
        }
        enemyList.Clear();
        camera.StopFollow();
        parent.SetActive(false);
        platform.SetActive(false);
        backPanel.DOFade(0, 0.1f);
    }
}
