﻿using UnityEngine;
using System.Collections.Generic;

public class EnemyController : Character
{
    bool initialized = false;
    public float rotationSpeed = 1;
    public float movementSpeed;
    public float damage;
    public float fireRate;

    public float attackRange = 40;

    List<Transform> playerTeam;
    List<Transform> tempList;

    public WeaponControl weapon;
    Rigidbody rigidBody;

    Transform closestPlayer = null;

    public CHARTYPE hostile;

    float timeToFindNext;
    public float timeToNext = 3;

    void Awake()
    {
        rigidBody = GetComponent<Rigidbody>();
        tempList = new List<Transform>();
    }

    public void Init(float h, float mS, float d, float f, List<Transform> l)
    {
        health = h;
        movementSpeed = mS;
        damage = d;
        fireRate = f;
        weapon.Init(damage, fireRate, hostile);
        playerTeam = l;
        closestPlayer = null;
        timeToFindNext = Time.time;
        initialized = true;
    }

    void FixedUpdate()
    {
        if (initialized && playerTeam.Count > 0)
        {
            rigidBody.velocity = Vector3.zero;
            if (closestPlayer == null || Time.time > timeToFindNext)
            {
                closestPlayer = ClosestPlayerTeamMember();
                if (closestPlayer == null)
                {
                    return;
                }
                timeToFindNext = Time.time + Random.Range(0.2f, timeToNext);
            }

            var q = Quaternion.LookRotation(closestPlayer.position - transform.position);
            transform.rotation = Quaternion.RotateTowards(transform.rotation, q, rotationSpeed * Time.deltaTime);
            if (DistanceToMe(closestPlayer.position) < attackRange)
            {
                Attack(closestPlayer);
            }
            else
            {
                rigidBody.velocity = transform.forward * movementSpeed;
            }
        }
        
    }

    void Attack(Transform t)
    {
        weapon.Attack();
    }

    Transform ClosestPlayerTeamMember(int num = 5)
    {
        if (playerTeam.Count == 0)
        {
            return null;
        }
        tempList.Clear();
        for (int i = 0; i < playerTeam.Count; i++)
        {
            if (playerTeam[i] != null)
            {
                tempList.Add(playerTeam[i]);
            }
        }
        if (tempList.Count == 0)
        {
            return null;
        }
        tempList.Sort(delegate(Transform t1, Transform t2)
            {
                return DistanceToMe(t1.position).CompareTo(DistanceToMe(t2.position));
            });
        int l = num >= tempList.Count ? tempList.Count - 1 : num;
        int r = Random.Range(0, l);
        return tempList[r];
    }

    float DistanceToMe(Vector3 pos)
    {

        return (transform.position - pos).sqrMagnitude;
    }
}
