﻿using UnityEngine;
using System.Collections;

public class AIMovement : MonoBehaviour {

    GameObject[] enemies;
    GameObject closestEnemy;
    GameObject[] closestFive;
    bool initialized = false;
    float speed;
    public bool isAttacking = false;
    public static AIMovement Instance;

    void Awake ()
    {
        Instance = this;
    }

    public void Init(float minSpeed, float maxSpeed, int playerNumber, int enemyNumber)
    {
        initialized = true;
        closestEnemy = null;
        closestFive = new GameObject[5];
        transform.GetChild(1).gameObject.SetActive(false);
        speed = Random.Range(minSpeed, maxSpeed) * Time.deltaTime;        
    }
    
    void FixedUpdate()
    {
        if (initialized && GameManager.Instance.enemyCount > 0 && GameManager.Instance.playerCount > 0)  
        {
            closestEnemy = FindClosest();
            if (Vector3.Distance(closestEnemy.transform.position, transform.position) < 2.0f && !isAttacking)
            {
                transform.GetChild(1).gameObject.SetActive(true);
                isAttacking = true;
                gameObject.GetComponent<Rigidbody>().velocity = new Vector3(Random.Range(-0.5f,0.5f),0, Random.Range(-0.5f, 0.5f));
                StartCoroutine("AttackAnim");
            }
            else
            {

                transform.LookAt(closestEnemy.transform);
                transform.rotation = Quaternion.Slerp(transform.rotation, Quaternion.LookRotation(closestEnemy.transform.position - transform.position), 50 * Time.deltaTime);
                if (gameObject.transform.tag == "PlayerTeam")
                    gameObject.GetComponent<Rigidbody>().velocity = transform.forward * speed;
                else
                    gameObject.GetComponent<Rigidbody>().velocity = transform.forward * speed;
            }
        }

        else
        {
            GameManager.Instance.KillGame();
        }
     }


    GameObject FindClosest()
    {
        if (gameObject.transform.tag == "PlayerTeam")
        enemies = GameObject.FindGameObjectsWithTag("EnemyTeam");
        else
           enemies = GameObject.FindGameObjectsWithTag("PlayerTeam");

        for (int j = 0; j < 5; j++)
        {
             float distance = Mathf.Infinity;
             Vector3 position = transform.position;
             foreach(GameObject enemy in enemies)
              {
                  Vector3 difference = enemy.transform.position - position;
                 float diff = difference.sqrMagnitude;
                 if (diff < distance)
                    {
                         if (closestFive[j] == enemy)
                             continue;
                          else
                             closestFive[j] = enemy;

                         distance = diff;
                     }

             }
        }

        int i = Random.Range(0,4);
        return closestFive[i];
        
    }

    IEnumerator AttackAnim()
    {
        yield return new WaitForSeconds(0.1f);
        isAttacking = false;
        transform.GetChild(1).gameObject.SetActive(false);
        yield return new WaitForSeconds(2.0f);
    }


    //void OnTriggerEnter(Collider hit)
    //{
    //    //if (this.gameObject.transform.GetChild(1).tag == "PlayerTeamDanda" && hit.gameObject.tag == "EnemyTeam")
    //    //{
    //    //    hit.gameObject.GetComponent<AIMovement>().health -= damage;


    //    //}

    //    //if (this.gameObject.transform.GetChild(1).tag == "EnemyTeamDanda" && hit.gameObject.tag == "PlayerTeam")
    //    //{
    //    //    if(hit.gameObject.transform.GetChild(0).tag == "PlayerTeamAnda")
    //    //    hit.gameObject.GetComponent<AIMovement>().health -= damage;

    //    //}

    //    if (isAttacking && this.gameObject.tag=="EnemyTeam" && hit.gameObject.tag == "PlayerTeam" && hit.gameObject.transform.GetChild(0).tag != "PlayerKaAnda")
    //    {
    //        hit.gameObject.GetComponent<AIMovement>().health -= damage;
    //    }
    //    if (isAttacking && this.gameObject.tag == "PlayerTeam" && hit.gameObject.tag == "EnemyTeam")
    //    {
    //        hit.gameObject.GetComponent<AIMovement>().health -= damage;
    //    }
    //    if (isAttacking && this.gameObject.tag == "EnemyTeam" && hit.gameObject.transform.GetChild(0).tag == "PlayerKaAnda")
    //    {
    //        hit.gameObject.GetComponent<PlayerMovement>().health -= damage;
    //    }

    //}


}
