﻿using UnityEngine;
using UnityEngine.UI;
using DG.Tweening;
using System.Collections;
using System.Collections.Generic;

[System.Serializable]
public class ConversationConfig
{
    public string playerName;
}

public class ConversationModule : MonoBehaviour
{
    public Image curtain;
    public Text currentText;
    public Image titleImage;

    public List<string> chatList;

    bool isAnimatingText = false;

    public float delayTime;
    public float delayTimeMultiplier;
    public float waitBetweenChats;

    int currentChat = -1;

    public ConversationConfig config;
    public ConversationConfig defaultConfig;

    string playerName = "";

    void Awake()
    {
        Reset();
    }

    public void StartConversation(ConversationConfig cfg = null)
    {
        Reset();
        gameObject.SetActive(true);
        if (cfg == null)
        {
            config = defaultConfig;
        }
        else
        {
            config = cfg;
        }
        FadeOutPanel();
        playerName = PlayerPrefs.GetString("PlayerName");
    }

    void FadeOutPanel()
    {
        var seq = DOTween.Sequence();
        seq.Append(titleImage.DOFade(1, 0.25f));
        seq.Append(curtain.DOFade(0, 2));
        seq.AppendInterval(2);
        seq.Append(titleImage.DOFade(0, 2));
        seq.AppendCallback(() =>
            {
                ShowCurrentChat();
            });
    }

    void DoneChatting()
    {
        var seq = DOTween.Sequence();
        seq.Append(curtain.DOFade(1, 3));
        seq.AppendCallback(() =>
            {
                End();       
            });
    }

    void FixedUpdate()
    {
        if (isAnimatingText)
        {
            if (Input.GetKey(KeyCode.Space))
            {
                delayTimeMultiplier = 0.01f;
            }
            else
            {
                delayTimeMultiplier = 1;
            }
        }
    }

    void End()
    {
        StateManager.instance.StartSacrifice();
        gameObject.SetActive(false);
    }

    void ShowCurrentChat()
    {
        currentChat += 1;
        if (currentChat >= chatList.Count)
        {
            DoneChatting();
            return;
        }

        string text = chatList[currentChat].Replace("GANDU", playerName);
        var seq = DOTween.Sequence();
        seq.Append(currentText.DOFade(0, delayTime * delayTimeMultiplier / 2));
        seq.AppendCallback(() =>
            {
                StartCoroutine(AnimateText(text));
            });
        seq.Append(currentText.DOFade(1, delayTime * delayTimeMultiplier / 2));

    }

    void Reset()
    {
        var colorCurtain = curtain.color;
        curtain.color = new Color(colorCurtain.r, colorCurtain.g, colorCurtain.b, 1);
        currentText.text = "";
        currentChat = -1;
        gameObject.SetActive(false);
    }

    void DoneAnimatingText()
    {
        ShowCurrentChat();
    }


    IEnumerator AnimateText(string strComplete)
    {

        int i = 0;
        isAnimatingText = true;
        currentText.text = "";
        while (i < strComplete.Length)
        {

            currentText.text += strComplete[i++];
            yield return new WaitForSeconds(delayTime * delayTimeMultiplier);
        }
        isAnimatingText = false;
        yield return new WaitForSeconds(waitBetweenChats * delayTimeMultiplier);
        DoneAnimatingText();
        yield return null;
    }
}
