﻿using UnityEngine;
using UnityEngine.UI;
using DG.Tweening;
using System.Collections.Generic;
using System.Collections;

[System.Serializable]
public class ChatQuestion
{
    public string chat;
    public bool expectAnswer;
    public float waitTime;
    public string setName;
}

public class IntroModule : MonoBehaviour
{
    public enum STATES
    {

        InitialQuestion,
        WaitingForName,
        Conversation,
        WaitingForAnswer
    }

    public enum ANSWERS
    {
        Yes,
        No
    }

    public Text mainText;
    public Image curtain;
    public InputField inputName;

    public List<ChatQuestion> chatList;

    public float delayTime;
    public float delayTimeMultiplier = 1;
    public float waitBetweenSentences = 2;

    public STATES currentState = STATES.InitialQuestion;
    public string playerName;

    public bool isAnimatingText = false;
    public int currentQuestion = 0;

    CanvasGroup inputNameCanvasGroup;

    void Awake()
    {
        inputNameCanvasGroup = inputName.GetComponent<CanvasGroup>();
    }

    void Start()
    {
    }

    public void StartIntro()
    {
        Reset();
        gameObject.SetActive(true);
        FadeOutPanel();
    }

    void AskInitialQuestion()
    {
        StartCoroutine(AnimateText(chatList[0].chat));
    }

    void DoneAnimatingText()
    {
        if (currentState == STATES.InitialQuestion)
        {
            GetName();
            currentQuestion = 0;
        }

        if (currentState == STATES.Conversation)
        {
            if (chatList[currentQuestion].expectAnswer)
            {
                GetAnswer();
            }
            else
            {
                var seq = DOTween.Sequence();
                seq.AppendInterval(chatList[currentQuestion].waitTime);
                seq.AppendCallback(() =>
                    {
                        AskQuestions();
                    });
            }
        }
    }

    void GetAnswer()
    {
        currentState = STATES.WaitingForAnswer;
    }

    void AnswerGiven(ANSWERS ans)
    {
        ProcessAnswer(ans);
        AskQuestions();
    }

    void ProcessAnswer(ANSWERS ans)
    {
        string currentSetName = chatList[currentQuestion].setName;
        if (currentSetName != "")
        {
            PlayerPrefs.SetInt(currentSetName, ans == ANSWERS.Yes ? 1 : 0);
        }
    }

    void StartConversation()
    {
        currentState = STATES.Conversation;
        inputName.enabled = false;
        mainText.text = "";
        var seq = DOTween.Sequence();
        seq.Append(inputNameCanvasGroup.DOFade(0, 0.5f));
        seq.AppendCallback(() =>
            {
                AskQuestions();
            });
    }

    void AskQuestions()
    {
        currentQuestion += 1;
        currentState = STATES.Conversation;
        if (currentQuestion >= chatList.Count)
        {
            DoneAskingQuestions();
            return;
        }
        string text = chatList[currentQuestion].chat.Replace("GANDU", playerName);
        var seq = DOTween.Sequence();
        seq.Append(mainText.DOFade(0, delayTime * delayTimeMultiplier / 2));
        seq.AppendCallback(() =>
            {
                StartCoroutine(AnimateText(text));
            });
        seq.Append(mainText.DOFade(1, delayTime * delayTimeMultiplier / 2));
    }

    void DoneAskingQuestions()
    {
        var seq = DOTween.Sequence();
        seq.Append(curtain.DOFade(1, 2));
        seq.AppendCallback(() =>
            {
                End();
            });
    }

    void End()
    {
        PlayerPrefs.SetString("PlayerName", playerName);
        StateManager.instance.StartConversation(playerName);
    }

    void GetName()
    {
        var seq = DOTween.Sequence();
        seq.Append(mainText.DOFade(0, 0.5f));
        seq.Append(inputNameCanvasGroup.DOFade(1, 0.2f));
        seq.AppendCallback(() =>
            {
                inputName.enabled = true;
                inputName.Select();
                inputName.ActivateInputField();
            });
        currentState = STATES.WaitingForName;
    }

    void FadeOutPanel()
    {
        var sequence = DOTween.Sequence();
        sequence.AppendInterval(0.5f);
        sequence.Append(curtain.DOFade(0, 1));
        sequence.AppendCallback(() =>
            {
                AskInitialQuestion();
            });

    }

    public void OnEnterName(string name)
    {
        playerName = inputName.text;
        StartConversation();
    }

    void LateUpdate()
    {
        if (isAnimatingText)
        {
            if (Input.GetKey(KeyCode.Space))
            {

                delayTimeMultiplier = 0.2f;
            }
            else
            {
                delayTimeMultiplier = 1;
            }
        }
        if (currentState == STATES.WaitingForAnswer)
        {
            if (Input.GetKeyDown(KeyCode.Y))
            {
                AnswerGiven(ANSWERS.Yes);
            }
            if (Input.GetKeyDown(KeyCode.N))
            {
                AnswerGiven(ANSWERS.No);
            }

        }
        if (currentState == STATES.WaitingForName)
        {
            if (inputName.isFocused == false)
            {
                // inputName.Select();
                inputName.ActivateInputField();
            }
        }
    }

    public void Reset()
    {
        gameObject.SetActive(false);
        mainText.text = "";
        var colorCurtain = curtain.color;
        curtain.color = new Color(colorCurtain.r, colorCurtain.g, colorCurtain.b, 1);
        currentState = STATES.InitialQuestion;
        inputName.enabled = false;
        inputNameCanvasGroup.alpha = 0;
        currentQuestion = 0;
    }

    IEnumerator AnimateText(string strComplete)
    {

        int i = 0;
        mainText.text = "";
        isAnimatingText = true;
        while (i < strComplete.Length)
        {

            mainText.text += strComplete[i++];
            yield return new WaitForSeconds(delayTime * delayTimeMultiplier);
        }
        yield return new WaitForSeconds(waitBetweenSentences);
        DoneAnimatingText();
        isAnimatingText = false;
        yield return null;
    }
}
