﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using DG.Tweening;

public enum AnimalList
{
    wife,
    child,
    chicken,
    cow,
    pow,
    puppy,
    dog,
    cat,
    soldier,
    oldie
}

[System.Serializable]
public class Level
{
    public string setName;
    public AnimalList[] list;
    public int[] people;
    public int[] morePeople;
    public int[] lessPeople;
    public int[] enemyCount;
}


public class StateManager : MonoBehaviour
{
    public int versionNumber = 0;

    public string winText;

    public enum GAMESTATE
    {
        Intro,
        Conversation,
        Sacrifice,
        Battle
    }

    public static StateManager instance;

    public IntroModule intro;
    public ConversationModule conversation;
    public SacrificeModule sacrifice;
    public BattleManager battle;

    public GAMESTATE currentState;

    int currentLevel = 0;

    public Level[] levels;

    public Image panel;
    public Text text;

    void Awake()
    {
        instance = this;
        int t = PlayerPrefs.GetInt("DONEINTRO" + versionNumber, 0);
        currentState = GAMESTATE.Intro;
        if (t != 0)
        {
            currentState = GAMESTATE.Conversation;
        }
        panel.DOFade(0, 0.1f);
        text.DOFade(0, 0.1f);
    }

    void Start()
    {
        if (currentState == GAMESTATE.Intro)
        {
            intro.StartIntro();
        }
        else
        {
            conversation.StartConversation();
        }
    }

    public void StartConversation(string pName)
    {
        PlayerPrefs.SetInt("DONEINTRO" + versionNumber, 1);
        intro.Reset();
        conversation.StartConversation();
    }

    public void StartSacrifice()
    {
        currentState = GAMESTATE.Sacrifice;
        Debug.Log("HERe");
        var cL = levels[currentLevel];
        sacrifice.StartSacrifice(GetSacrificId(cL));
    }

    AnimalList GetSacrificId(Level l)
    {
        var animalList = l.list;
        int r = Random.Range(0, l.list.Length);
        return animalList[r];
    }

    public void StartBattle(bool didSacrifice)
    {
        currentState = GAMESTATE.Battle;
        var level = levels[currentLevel];
        BattleData bfg = new BattleData();
        bfg.totalAllies = Random.Range(level.people[0], level.people[1]);
        int initial = bfg.totalAllies;
        if (didSacrifice)
        {
            bfg.totalAllies += Random.Range(level.morePeople[0], level.morePeople[1]);
        }
        else
        {
            bfg.totalAllies -= Random.Range(level.lessPeople[0], level.lessPeople[1]);
        }
        bfg.totalEnemies = Random.Range(level.enemyCount[0], level.enemyCount[1]);
        bfg.people = initial;
        bfg.change = initial - bfg.totalAllies;
        battle.StartBattle(bfg);
    }

    public void EndBattle(bool won)
    {
        if (won)
        {
            currentLevel += 1;
            Debug.Log(currentLevel);
            Debug.Log(levels.Length);
            if (currentLevel == levels.Length)
            {
                GameOver();
            }
            else
            {
                StartSacrifice();
            }
        }
        else
        {
            StartSacrifice();
        }
    }

    void GameOver() // this is the final page
    {
        text.text = "";
        var seq = DOTween.Sequence();
        seq.Join(panel.DOFade(1, 1f));
        seq.Join(text.DOFade(1, 1f));
        seq.AppendCallback(() =>
            {
                StartCoroutine(AnimateText(winText));
            });
    }

    IEnumerator AnimateText(string strComplete)
    {

        int i = 0;
        text.text = "";
        while (i < strComplete.Length)
        {

            text.text += strComplete[i++];
            yield return new WaitForSeconds(0.1f);
        }
        yield return new WaitForSeconds(2);
        DoneAnimatingText();
        yield return null;
    }

    void DoneAnimatingText()
    {
        
    }

}
