﻿using UnityEngine;
using DG.Tweening;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

[System.Serializable]
public class AnimalDetails
{
    public string setName;
    public AnimalList id;
    public DeadThing details;
    public List<string> happyChatList;
    public List<string> sadChatList;
}

public class SacrificeModule : MonoBehaviour
{
    public enum STATES
    {
        Nothing,
        Conversation,
        Question,
        Answered
    }

    public enum ANSWER
    {
        Yes,
        No
    }

    public Image curtain;
    public Text currentText;
    public Image TheFlash;
    public Image Noise;
    public Image thing;

    int currentChatNumber = -1;
    public SacrificeConfig config;
    public SacrificeConfig defaultConfig;

    public float delayTimeMultiplier = 1;

    public STATES currentState = STATES.Nothing;

    public AnimalDetails[] details;

    bool didDoSacrifice = false;

    void Awake()
    {
        Reset();
    }

    void SetThing()
    {
        thing.sprite = config.thing.alterImage;
    }

    void StartNoise()
    {
        var color = Noise.color;
        Noise.color = new Color(color.r, color.g, color.b, 0.2f);
        var seq = DOTween.Sequence();
        seq.Append(Noise.DOFade(0.4f, 5).SetEase(Ease.InSine));
        seq.Append(Noise.DOFade(0.2f, 5).SetEase(Ease.OutSine));
        seq.SetLoops(-1);
    }

    void LateUpdate()
    {
        if (currentState == STATES.Conversation)
        {
            if (Input.GetKey(KeyCode.Space))
            {

                delayTimeMultiplier = 0.2f;
            }
            else
            {
                delayTimeMultiplier = 1;
            }
        }
        if (currentState == STATES.Question)
        {
            if (Input.GetKeyDown(KeyCode.Y))
            {
                AnswerQuestion(ANSWER.Yes);
            }
            if (Input.GetKeyDown(KeyCode.N))
            {
                AnswerQuestion(ANSWER.No);
            }
        }
    }

    void AnswerQuestion(ANSWER ans)
    {
        currentState = STATES.Answered;
        if (ans == ANSWER.Yes)
        {
            DoSacrifice();
        }
        else
        {
            EndSacrifice();
        }
    }

    void DoSacrifice()
    {
        didDoSacrifice = true;
        var seq = DOTween.Sequence();
        seq.Append(TheFlash.DOFade(1, 0.5f).SetEase(config.currentEase));
        seq.AppendCallback(() =>
            {
                thing.sprite = config.thing.deadImage;
            });
        seq.Append(TheFlash.DOFade(0, 2));
        seq.AppendInterval(2);
        seq.AppendCallback(() =>
            {
                EndSacrifice();
            });
    }

    void EndSacrifice()
    {
        var seq = DOTween.Sequence();
        seq.Append(curtain.DOFade(1, 3));
        seq.AppendCallback(() =>
            {
                End();       
            });
    }

    void End()
    {
        StateManager.instance.StartBattle(didDoSacrifice);
        gameObject.SetActive(false);
    }

    void SetupDetails(AnimalList l)
    {
        AnimalDetails d = new AnimalDetails();
        for (int i = 0; i < details.Length; i++)
        {
            if (details[i].id == l)
            {
                d = details[i];
            }
        }

        int isHappy = PlayerPrefs.GetInt(d.setName);
        if (isHappy == 1)
        {
            config.chatConfig = d.happyChatList;
        }
        else
        {
            config.chatConfig = d.sadChatList;
        }
        config.generalName = PlayerPrefs.GetString("PlayerName");
        config.thing = d.details;
    }

    public void StartSacrifice(AnimalList l)
    {
        Reset();
        gameObject.SetActive(true);
        StartNoise();
        currentState = STATES.Conversation;
        config = defaultConfig;
        SetupDetails(l);
        SetThing();
        FadeInCanvas();

    }

    void FadeInCanvas()
    {
        var sequence = DOTween.Sequence();
        sequence.AppendInterval(config.initialDelay);
        sequence.Append(curtain.DOFade(0, config.initialFadeInTime));
        sequence.AppendCallback(() =>
            {
                StartConversation();
            });
    }

    void StartConversation()
    {
        ShowCurrentChat();
    }

    void ShowCurrentChat()
    {
        currentChatNumber += 1;
        if (currentChatNumber >= config.chatConfig.Count)
        {
            AskQuestion();
            return;
        }
        string text = config.chatConfig[currentChatNumber].Replace("GANDU", config.generalName);  
        var seq = DOTween.Sequence();
        seq.Append(currentText.DOFade(0, config.chatFadeInDelay * delayTimeMultiplier / 2));
        seq.AppendCallback(() =>
            {
                StartCoroutine(AnimateText(text));
            });
        seq.Append(currentText.DOFade(1, config.chatFadeInDelay * delayTimeMultiplier / 2));
    }

    void AskQuestion()
    {
        currentState = STATES.Question;
        var seq = DOTween.Sequence();
        seq.Append(currentText.DOFade(0, 1));
        seq.AppendInterval(2 * delayTimeMultiplier);
        seq.AppendCallback(() =>
            {
                currentText.text = "Sacrifice " + config.thing.name + "? (Y/N)";
            });
        seq.Append(currentText.DOFade(1, 2));
    }

    void Reset()
    {
        gameObject.SetActive(false);
        currentChatNumber = -1;
        currentText.text = "";
        currentState = STATES.Nothing;
        didDoSacrifice = false;
        var color = TheFlash.color;
        TheFlash.color = new Color(color.r, color.g, color.b, 0);
        var colorCurtain = curtain.color;
        curtain.color = new Color(colorCurtain.r, colorCurtain.g, colorCurtain.b, 1);
    }

    void DoneAnimatingText()
    {
        ShowCurrentChat();
    }

    IEnumerator AnimateText(string strComplete)
    {

        int i = 0;
        currentText.text = "";
        while (i < strComplete.Length)
        {

            currentText.text += strComplete[i++];
            yield return new WaitForSeconds(config.delayTimeInText * delayTimeMultiplier);
        }
        yield return new WaitForSeconds(config.waitBetweenChats * delayTimeMultiplier);
        DoneAnimatingText();
        yield return null;
    }
}
